﻿Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.DesktopEdition
Imports System.Collections.ObjectModel
Public Class Form1
   Private moLine As LineShape
   Private moPolygon As PolygonShape
   Private moMap As WinformsMap
   Private moShapefile As MyShapefile
   Private moInMemoryFeatureLayer As New InMemoryFeatureLayer
   Private WithEvents moTrackLayer As TrackInteractiveOverlay
   'Private WithEvents moEditOverlay As MyEditOverlay
   Private msKey As String
   Private moEditFeatures As New Collection(Of Feature)
   Public Const cdMICRON As Double = 0.000001
   Private msInstructions As String =
      "1. Select a large number (the group in the upper left should do it) of features by clicking the 'Select Features' radio button and using a track rectangle." & ControlChars.CrLf &
      "2. Delete features using the delete key." & ControlChars.CrLf &
      "3. Notice the errors that are returned in the transaction result." & ControlChars.CrLf
   Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
      Dim oLayerOverlay As New LayerOverlay
      Dim oSelectionOverlay As New LayerOverlay

      Me.TextBox1.Text = msInstructions
      moMap = WinformsMap1
      moMap.MapUnit = GeographyUnit.Meter
      moMap.MinimumScale = 0
      ShapeFileFeatureLayer.Rebuild("..\..\Data\CruisePlots.shp")
      moShapefile = New MyShapefile("..\..\Data\CruisePlots.shp", ShapeFileReadWriteMode.ReadWrite)

      moShapefile.Name = "CruisePlots"
      oLayerOverlay.Layers.Add("Shapefile", moShapefile)

      Dim oPointStyle As PointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.SimpleColors.Red, 8)
      moShapefile.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(oPointStyle)
      moShapefile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

      moShapefile.Open()

      moMap.Overlays.Add(oLayerOverlay)
      moMap.TrackOverlay.TrackMode = TrackMode.Rectangle

      moTrackLayer = WinformsMap1.TrackOverlay

      WinformsMap1.CurrentExtent = moShapefile.GetBoundingBox
      WinformsMap1.Refresh()
   End Sub

   Private Sub WinformsMap1_KeyUp(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles WinformsMap1.KeyUp
      mDeleteSelectedFeatures()

   End Sub
   Private Sub mDeleteSelectedFeatures()
      Dim oSelectionLayer As InMemoryFeatureLayer = moMap.EditOverlay.EditShapesLayer
      Dim oBuilder As New System.Text.StringBuilder
      If oSelectionLayer.InternalFeatures.Count > 0 Then
         If moShapefile.IsOpen Then
            moShapefile.Open()
         End If
         moShapefile.Open()
         moShapefile.EditTools.BeginTransaction()
         For Each oFeature As Feature In oSelectionLayer.InternalFeatures
            moShapefile.EditTools.Delete(oFeature.Id)
         Next oFeature
         Dim oTransResult As TransactionResult = moShapefile.EditTools.CommitTransaction()
         moShapefile.Close()
         oSelectionLayer.InternalFeatures.Clear()
         ShapeFileFeatureLayer.Rebuild(moShapefile.ShapePathFileName)
         If oTransResult.TransactionResultStatus = TransactionResultStatus.Failure Then
            oBuilder.Append("Commit Failed:" & ControlChars.CrLf)
            oBuilder.Append("Total Success Count: " & oTransResult.TotalSuccessCount.ToString & ControlChars.CrLf)
            oBuilder.Append("Total Failure Count: " & oTransResult.TotalFailureCount.ToString & ControlChars.CrLf)
            For Each oKvp In oTransResult.FailureReasons
               oBuilder.Append(oKvp.Value & ControlChars.CrLf)
            Next oKvp
            Dim oErrForm As New frmError
            oErrForm.txtError.Text = oBuilder.ToString
            oErrForm.Show()
         End If
         WinformsMap1.Refresh()
      End If
   End Sub

   Private Sub mSelecteFeatures(ByVal oRectangle As RectangleShape)
      moMap.EditOverlay.EditShapesLayer.InternalFeatures.Clear()

      '***** Select a feature
      moShapefile.Open()
      Dim oFeatures As Collection(Of Feature) = moShapefile.QueryTools.GetFeaturesWithin(oRectangle, ReturningColumnsType.AllColumns)
      If oFeatures.Count > 0 Then
         With WinformsMap1.EditOverlay.EditShapesLayer
            .Open()
            .EditTools.IsTransactionLive = True
            .EditTools.BeginTransaction()
            For Each oFeature As Feature In oFeatures
               .EditTools.Add(oFeature)
            Next oFeature
            .EditTools.CommitTransaction()
         End With
      End If
      moShapefile.Close()
      WinformsMap1.Refresh()
   End Sub

   Private Sub moTrackLayer_TrackEnded(ByVal sender As Object, ByVal e As ThinkGeo.MapSuite.DesktopEdition.TrackEndedTrackInteractiveOverlayEventArgs) Handles moTrackLayer.TrackEnded
      moTrackLayer.TrackShapeLayer.InternalFeatures.Clear()
      mSelecteFeatures(CType(e.TrackShape, RectangleShape))
      WinformsMap1.Refresh()
   End Sub

   Private Sub mCommitTransaction()
      If Not moShapefile.IsOpen Then
         moShapefile.Open()
      End If

      If moShapefile.EditTools.IsInTransaction Then
         moShapefile.EditTools.CommitTransaction()
         moShapefile.Close()
         WinformsMap1.Refresh()
      End If

   End Sub

   Private Sub rbPan_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles rbPan.CheckedChanged, rbSelectFeatures.CheckedChanged
      If Not moMap Is Nothing Then
         If rbPan.Checked Then
            moMap.TrackOverlay.TrackMode = TrackMode.None
         Else
            moMap.TrackOverlay.TrackMode = TrackMode.Rectangle
         End If
      End If
   End Sub
End Class
